#include "stdafx.h"
#include <shlwapi.h>
#include "HDUSTest.h"
#include "ColorScheme.h"
#include "Settings.h"
#include "resource.h"




const CColorScheme::ColorInfo CColorScheme::m_ColorInfoList[NUM_COLORS] = {
	{RGB(128,192,160),	TEXT("StatusBack"),					TEXT("Xe[^Xo[wi")},
	{RGB(64,96,80),		TEXT("StatusText"),					TEXT("Xe[^Xo[")},
	{RGB(64,96,80),		TEXT("StatusHighlightBack"),		TEXT("Xe[^Xo[Iwi")},
	{RGB(128,192,160),	TEXT("StatusHighlightText"),		TEXT("Xe[^Xo[I𕶎")},
	{RGB(128,192,160),	TEXT("PanelBack"),					TEXT("plwi")},
	{RGB(64,96,80),		TEXT("PanelText"),					TEXT("pl")},
	{RGB(238,238,238),	TEXT("PanelTabBack"),				TEXT("pl^uwi")},
	{RGB(85,85,85),		TEXT("PanelTabText"),				TEXT("pl^u")},
	{RGB(85,85,85),		TEXT("PanelTabBorder"),				TEXT("pl^ug")},
	{RGB(128,192,160),	TEXT("PanelCurTabBack"),			TEXT("plI^uwi")},
	{RGB(64,96,80),		TEXT("PanelCurTabText"),			TEXT("plI^u")},
	{RGB(64,96,80),		TEXT("PanelCurTabBorder"),				TEXT("plI^ug")},
	{RGB(238,238,238),	TEXT("PanelTabMargin"),				TEXT("pl^u]")},
	{RGB(170,170,170),	TEXT("PanelTitleBack"),				TEXT("pl^Cgwi")},
	{RGB(0,0,0),		TEXT("PanelTitleText"),				TEXT("pl^Cg")},
	{RGB(128,192,160),	TEXT("ProgramInfoBack"),			TEXT("ԑgwi")},
	{RGB(64,96,80),		TEXT("ProgramInfoText"),			TEXT("ԑg񕶎")},
	{RGB(128,192,160),	TEXT("ProgramListBack"),			TEXT("ԑg\wi")},
	{RGB(64,96,80),		TEXT("ProgramListText"),			TEXT("ԑg\")},
	{RGB(64,96,80),		TEXT("ProgramListTitleBack"),		TEXT("ԑg\ԑgwi")},
	{RGB(128,192,160),	TEXT("ProgramListTitleText"),		TEXT("ԑg\ԑg")},
	{RGB(64,96,80),		TEXT("ControlPanelHighlightBack"),	TEXT("plIwi")},
	{RGB(128,192,160),	TEXT("ControlPanelHighlightText"),	TEXT("plI𕶎")},
};


CColorScheme::CColorScheme()
{
	SetDefault();
	m_pszName=NULL;
	m_pszFileName=NULL;
}


CColorScheme::~CColorScheme()
{
	delete [] m_pszName;
	delete [] m_pszFileName;
}


COLORREF CColorScheme::GetColor(int Type) const
{
	if (Type<0 || Type>=NUM_COLORS)
		return CLR_INVALID;
	return m_ColorList[Type];
}


bool CColorScheme::SetColor(int Type,COLORREF Color)
{
	if (Type<0 || Type>=NUM_COLORS)
		return false;
	m_ColorList[Type]=Color;
	return true;
}


bool CColorScheme::SetName(LPCTSTR pszName)
{
	delete [] m_pszName;
	m_pszName=DuplicateString(pszName);
	return true;
}


bool CColorScheme::Load(LPCTSTR pszFileName)
{
	CSettings Settings;
	TCHAR szName[128];
	int i;

	if (!Settings.Open(pszFileName,TEXT("ColorScheme"),CSettings::OPEN_READ))
		return false;
	if (Settings.Read(TEXT("Name"),szName,lengthof(szName)))
		SetName(szName);
	for (i=0;i<NUM_COLORS;i++) {
		Settings.ReadColor(m_ColorInfoList[i].pszText,&m_ColorList[i]);
	}
	SetFileName(pszFileName);
	return true;
}


bool CColorScheme::Save(LPCTSTR pszFileName) const
{
	CSettings Settings;
	int i;

	if (!Settings.Open(pszFileName,TEXT("ColorScheme"),CSettings::OPEN_WRITE))
		return false;
	Settings.Write(TEXT("Name"),m_pszName!=NULL?m_pszName:TEXT(""));
	for (i=0;i<NUM_COLORS;i++) {
		Settings.WriteColor(m_ColorInfoList[i].pszText,m_ColorList[i]);
	}
	return true;
}


bool CColorScheme::SetFileName(LPCTSTR pszFileName)
{
	delete [] m_pszFileName;
	m_pszFileName=DuplicateString(pszFileName);
	return true;
}


void CColorScheme::SetDefault()
{
	int i;

	for (i=0;i<NUM_COLORS;i++)
		m_ColorList[i]=m_ColorInfoList[i].DefaultColor;
}


LPCTSTR CColorScheme::GetColorName(int Type)
{
	if (Type<0 || Type>=NUM_COLORS)
		return NULL;
	return m_ColorInfoList[Type].pszName;
}


COLORREF CColorScheme::GetDefaultColor(int Type)
{
	if (Type<0 || Type>=NUM_COLORS)
		return CLR_INVALID;
	return m_ColorInfoList[Type].DefaultColor;
}




CColorSchemeList::CColorSchemeList()
{
	m_NumColorSchemes=0;
	m_ppList=NULL;
}


CColorSchemeList::~CColorSchemeList()
{
	Clear();
}


bool CColorSchemeList::Add(CColorScheme *pColorScheme)
{
	m_ppList=(CColorScheme**)realloc(m_ppList,(m_NumColorSchemes+1)*sizeof(CColorScheme*));
	m_ppList[m_NumColorSchemes++]=pColorScheme;
	return true;
}


bool CColorSchemeList::Load(LPCTSTR pszDirectory)
{
	HANDLE hFind;
	WIN32_FIND_DATA wfd;
	TCHAR szFileName[MAX_PATH];

	PathCombine(szFileName,pszDirectory,TEXT("*.httheme"));
	hFind=FindFirstFile(szFileName,&wfd);
	if (hFind!=INVALID_HANDLE_VALUE) {
		do {
			CColorScheme *pColorScheme;

			PathCombine(szFileName,pszDirectory,wfd.cFileName);
			pColorScheme=new CColorScheme;
			if (pColorScheme->Load(szFileName))
				Add(pColorScheme);
			else
				delete pColorScheme;
		} while (FindNextFile(hFind,&wfd));
	}
	return true;
}


void CColorSchemeList::Clear()
{
	if (m_ppList!=NULL) {
		int i;

		for (i=0;i<m_NumColorSchemes;i++)
			delete m_ppList[i];
		free(m_ppList);
		m_ppList=NULL;
		m_NumColorSchemes=0;
	}
}


CColorScheme *CColorSchemeList::GetColorScheme(int Index)
{
	if (Index<0 || Index>=m_NumColorSchemes)
		return NULL;
	return m_ppList[Index];
}




const LPCTSTR CColorSchemeOptions::m_pszExtension=TEXT(".httheme");


CColorSchemeOptions::CColorSchemeOptions()
{
	m_pColorScheme=new CColorScheme;
	m_pApplyFunc=NULL;
}


CColorSchemeOptions::~CColorSchemeOptions()
{
	delete m_pColorScheme;
}


bool CColorSchemeOptions::Load(LPCTSTR pszFileName)
{
	CSettings Settings;
	COLORREF Color;

	// Backward compatibility
	if (Settings.Open(pszFileName,TEXT("Settings"),CSettings::OPEN_READ)) {
		if (Settings.ReadColor(TEXT("StatusBkColor"),&Color))
			m_pColorScheme->SetColor(CColorScheme::COLOR_STATUSBACK,Color);
		if (Settings.ReadColor(TEXT("StatusTextColor"),&Color))
			m_pColorScheme->SetColor(CColorScheme::COLOR_STATUSTEXT,Color);
		if (Settings.ReadColor(TEXT("StatusHighlightBackColor"),&Color))
			m_pColorScheme->SetColor(CColorScheme::COLOR_STATUSHIGHLIGHTBACK,Color);
		if (Settings.ReadColor(TEXT("StatusHighlightTextColor"),&Color))
			m_pColorScheme->SetColor(CColorScheme::COLOR_STATUSHIGHLIGHTTEXT,Color);
		if (Settings.ReadColor(TEXT("InfoBackColor"),&Color)) {
			m_pColorScheme->SetColor(CColorScheme::COLOR_PANELBACK,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_PANELCURTABBACK,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_PROGRAMLISTBACK,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_PROGRAMLISTTITLETEXT,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_CONTROLPANELHIGHLIGHTTEXT,Color);
		}
		if (Settings.ReadColor(TEXT("InfoTextColor"),&Color)) {
			m_pColorScheme->SetColor(CColorScheme::COLOR_PANELTEXT,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_PANELCURTABTEXT,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_PROGRAMLISTTEXT,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_PROGRAMLISTTITLEBACK,Color);
			m_pColorScheme->SetColor(CColorScheme::COLOR_CONTROLPANELHIGHLIGHTBACK,Color);
		}
		Settings.Close();
	}
	m_pColorScheme->Load(pszFileName);
	return true;
}


bool CColorSchemeOptions::Save(LPCTSTR pszFileName) const
{
	return m_pColorScheme->Save(pszFileName);
}


bool CColorSchemeOptions::SetApplyCallback(ApplyFunc pCallback)
{
	m_pApplyFunc=pCallback;
	return true;
}


bool CColorSchemeOptions::ApplyColorScheme() const
{
	return Apply(m_pColorScheme);
}


bool CColorSchemeOptions::Apply(const CColorScheme *pColorScheme) const
{
	if (m_pApplyFunc==NULL)
		return false;
	return m_pApplyFunc(pColorScheme);
}


void CColorSchemeOptions::GetListColor(HWND hDlg,CColorScheme *pColorScheme)
{
	int i;

	for (i=0;i<CColorScheme::NUM_COLORS;i++)
		pColorScheme->SetColor(i,
				(COLORREF)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETITEMDATA,i,0));
}


void CColorSchemeOptions::SetListColor(HWND hDlg,const CColorScheme *pColorScheme)
{
	int i;

	for (i=0;i<CColorScheme::NUM_COLORS;i++)
		SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_SETITEMDATA,i,pColorScheme->GetColor(i));
}


bool CColorSchemeOptions::GetThemesDirectory(LPTSTR pszDirectory,int MaxLength,bool fCreate)
{
	GetModuleFileName(NULL,pszDirectory,MaxLength);
	lstrcpy(PathFindFileName(pszDirectory),TEXT("Themes"));
	if (fCreate && !PathIsDirectory(pszDirectory))
		CreateDirectory(pszDirectory,NULL);
	return true;
}


CColorSchemeOptions *CColorSchemeOptions::GetThis(HWND hwnd)
{
	return static_cast<CColorSchemeOptions*>(GetProp(hwnd,TEXT("This")));
}


BOOL CALLBACK CColorSchemeOptions::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			CColorSchemeOptions *pThis=reinterpret_cast<CColorSchemeOptions*>(lParam);
			TCHAR szDirectory[MAX_PATH];
			int i;
			int Index;

			SetProp(hDlg,TEXT("This"),pThis);
			GetThemesDirectory(szDirectory,lengthof(szDirectory));
			pThis->m_PresetList.Load(szDirectory);
			for (i=0;i<pThis->m_PresetList.NumColorSchemes();i++) {
				LPCTSTR pszName=pThis->m_PresetList.GetColorScheme(i)->GetName();

				Index=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_ADDSTRING,0,
									(LPARAM)(pszName!=NULL?pszName:TEXT("")));
				SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_SETITEMDATA,Index,i);
			}
			for (i=0;i<CColorScheme::NUM_COLORS;i++) {
				SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_ADDSTRING,0,pThis->m_pColorScheme->GetColor(i));
			}
			pThis->m_fPreview=false;

			RECT rc;
			static const RGBQUAD BaseColors[18] = {
				{0x00, 0x00, 0xFF},
				{0x00, 0x66, 0xFF},
				{0x00, 0xCC, 0xFF},
				{0x00, 0xFF, 0xFF},
				{0x00, 0xFF, 0xCC},
				{0x00, 0xFF, 0x66},
				{0x00, 0xFF, 0x00},
				{0x66, 0xFF, 0x00},
				{0xCC, 0xFF, 0x00},
				{0xFF, 0xFF, 0x00},
				{0xFF, 0xCC, 0x00},
				{0xFF, 0x66, 0x00},
				{0xFF, 0x00, 0x00},
				{0xFF, 0x00, 0x66},
				{0xFF, 0x00, 0xCC},
				{0xFF, 0x00, 0xFF},
				{0xCC, 0x00, 0xFF},
				{0x66, 0x00, 0xFF},
			};
			RGBQUAD Palette[256];
			int j,k;

			CColorPalette::Initialize(GetWindowInstance(hDlg));
			pThis->m_ColorPalette.Create(hDlg,WS_CHILD | WS_VISIBLE,0,IDC_COLORSCHEME_PALETTE);
			GetWindowRect(GetDlgItem(hDlg,IDC_COLORSCHEME_PALETTEPLACE),&rc);
			MapWindowPoints(NULL,hDlg,(LPPOINT)&rc,2);
			pThis->m_ColorPalette.SetPosition(&rc);
			for (i=0;i<lengthof(BaseColors);i++) {
				RGBQUAD Color=BaseColors[i%2*(lengthof(BaseColors)/2)+i/2];

				for (j=0;j<4;j++) {
					Palette[i*8+j].rgbBlue=(Color.rgbBlue*(j+1))/4;
					Palette[i*8+j].rgbGreen=(Color.rgbGreen*(j+1))/4;
					Palette[i*8+j].rgbRed=(Color.rgbRed*(j+1))/4;
				}
				for (;j<8;j++) {
					Palette[i*8+j].rgbBlue=Color.rgbBlue+(255-Color.rgbBlue)*(j-3)/5;
					Palette[i*8+j].rgbGreen=Color.rgbGreen+(255-Color.rgbGreen)*(j-3)/5;
					Palette[i*8+j].rgbRed=Color.rgbRed+(255-Color.rgbRed)*(j-3)/5;
				}
			}
			i=lengthof(BaseColors)*8;
			for (j=0;j<16;j++) {
				Palette[i].rgbBlue=(255*j)/15;
				Palette[i].rgbGreen=(255*j)/15;
				Palette[i].rgbRed=(255*j)/15;
				i++;
			}
			for (j=0;j<CColorScheme::NUM_COLORS;j++) {
				COLORREF cr=pThis->m_pColorScheme->GetColor(j);

				for (k=0;k<i;k++) {
					if (cr==RGB(Palette[k].rgbRed,Palette[k].rgbGreen,Palette[k].rgbBlue))
						break;
				}
				if (k==i) {
					Palette[i].rgbBlue=GetBValue(cr);
					Palette[i].rgbGreen=GetGValue(cr);
					Palette[i].rgbRed=GetRValue(cr);
					i++;
				}
			}
			if (i<lengthof(Palette))
				ZeroMemory(&Palette[i],(lengthof(Palette)-i)*sizeof(RGBQUAD));
			pThis->m_ColorPalette.SetPalette(Palette,lengthof(Palette));
		}
		return TRUE;

	case WM_MEASUREITEM:
		reinterpret_cast<LPMEASUREITEMSTRUCT>(lParam)->itemHeight=
											7*HIWORD(GetDialogBaseUnits())/8;
		return TRUE;

	case WM_DRAWITEM:
		{
			CColorSchemeOptions *pThis=GetThis(hDlg);
			LPDRAWITEMSTRUCT pdis=reinterpret_cast<LPDRAWITEMSTRUCT>(lParam);

			switch (pdis->itemAction) {
			case ODA_DRAWENTIRE:
			case ODA_SELECT:
				{
					HBRUSH hbr,hbrOld;
					HPEN hpenOld;
					RECT rc;
					int OldBkMode;
					COLORREF crOldTextColor;

					hbr=CreateSolidBrush(GetSysColor(
								(pdis->itemState&ODS_SELECTED)==0?COLOR_WINDOW:
															COLOR_HIGHLIGHT));
					FillRect(pdis->hDC,&pdis->rcItem,hbr);
					DeleteObject(hbr);
					hbr=CreateSolidBrush((COLORREF)pdis->itemData);
					hbrOld=(HBRUSH)SelectObject(pdis->hDC,hbr);
					hpenOld=(HPEN)SelectObject(pdis->hDC,GetStockObject(BLACK_PEN));
					rc.left=pdis->rcItem.left+2;
					rc.top=pdis->rcItem.top+2;
					rc.bottom=pdis->rcItem.bottom-2;
					rc.right=rc.left+(rc.bottom-rc.top)*2;
					Rectangle(pdis->hDC,rc.left,rc.top,rc.right,rc.bottom);
					SelectObject(pdis->hDC,hpenOld);
					SelectObject(pdis->hDC,hbrOld);
					DeleteObject(hbr);
					OldBkMode=SetBkMode(pdis->hDC,TRANSPARENT);
					crOldTextColor=SetTextColor(pdis->hDC,GetSysColor(
						(pdis->itemState&ODS_SELECTED)==0?COLOR_WINDOWTEXT:
														COLOR_HIGHLIGHTTEXT));
					rc.left=rc.right+2;
					rc.top=pdis->rcItem.top;
					rc.right=pdis->rcItem.right;
					rc.bottom=pdis->rcItem.bottom;
					DrawText(pdis->hDC,CColorScheme::GetColorName(pdis->itemID),-1,
						&rc,DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
					SetTextColor(pdis->hDC,crOldTextColor);
					SetBkMode(pdis->hDC,OldBkMode);
					if ((pdis->itemState&ODS_FOCUS)==0)
						break;
				}
			case ODA_FOCUS:
				DrawFocusRect(pdis->hDC,&pdis->rcItem);
				break;
			}
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_COLORSCHEME_PRESET:
			if (HIWORD(wParam)==CBN_SELCHANGE) {
				int Sel=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_GETCURSEL,0,0);

				if (Sel>=0) {
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int Index=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_GETITEMDATA,Sel,0);
					CColorScheme *pColorScheme=pThis->m_PresetList.GetColorScheme(Index);

					if (pColorScheme!=NULL) {
						SetListColor(hDlg,pColorScheme);
						SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_SETSEL,FALSE,-1);
						InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);
						pThis->m_ColorPalette.SetSel(-1);
						if (pThis->Apply(pColorScheme))
							pThis->m_fPreview=true;
					}
				}
			}
			return TRUE;

		case IDC_COLORSCHEME_SAVE:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);
				TCHAR szName[MAX_PATH],szFileName[MAX_PATH];
				int Index;
				CColorScheme *pColorScheme;
				LPCTSTR pszFileName;

				GetDlgItemText(hDlg,IDC_COLORSCHEME_PRESET,szName,lengthof(szName));
				if (szName[0]=='\0') {
					MessageBox(hDlg,TEXT("O͂ĂB"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				pszFileName=szName;
				Index=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_FINDSTRINGEXACT,-1,(LPARAM)szName);
				if (Index>=0) {
					pColorScheme=pThis->m_PresetList.GetColorScheme(
						SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_GETITEMDATA,Index,0));
					if (pColorScheme!=NULL)
						pszFileName=PathFindFileName(pColorScheme->GetFileName());
				}
				GetThemesDirectory(szFileName,lengthof(szFileName),true);
				if (lstrlen(szFileName)+1+lstrlen(pszFileName)+lstrlen(m_pszExtension)>=MAX_PATH) {
					MessageBox(hDlg,TEXT("O܂B"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				PathAppend(szFileName,pszFileName);
				if (pszFileName==szName)
					lstrcat(szFileName,m_pszExtension);
				pColorScheme=new CColorScheme;
				pColorScheme->SetName(szName);
				GetListColor(hDlg,pColorScheme);
				if (!pColorScheme->Save(szFileName)) {
					delete pColorScheme;
					MessageBox(hDlg,TEXT("ۑł܂B"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				if (Index<0) {
					pColorScheme->SetFileName(szFileName);
					pThis->m_PresetList.Add(pColorScheme);
					Index=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_ADDSTRING,0,(LPARAM)szName);
					SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_SETITEMDATA,Index,pThis->m_PresetList.NumColorSchemes()-1);
				} else
					delete pColorScheme;
			}
			return TRUE;

		case IDC_COLORSCHEME_DELETE:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);
				TCHAR szName[MAX_PATH],szFileName[MAX_PATH];
				int Index;
				CColorScheme *pColorScheme;
				LPCTSTR pszFileName;

				GetDlgItemText(hDlg,IDC_COLORSCHEME_PRESET,szName,lengthof(szName));
				if (szName[0]=='\0')
					break;
				Index=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_FINDSTRINGEXACT,-1,(LPARAM)szName);
				if (Index<0
						|| (pColorScheme=pThis->m_PresetList.GetColorScheme(
							SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_GETITEMDATA,Index,0)))==NULL)
					break;
				GetThemesDirectory(szFileName,lengthof(szFileName));
				pszFileName=PathFindFileName(pColorScheme->GetFileName());
				if (lstrlen(szFileName)+1+lstrlen(pszFileName)+lstrlen(m_pszExtension)>=MAX_PATH)
					break;
				PathAppend(szFileName,pszFileName);
				if (pszFileName==szName)
					lstrcat(szFileName,m_pszExtension);
				if (!DeleteFile(szFileName)) {
					MessageBox(hDlg,TEXT("t@C폜ł܂B"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_DELETESTRING,Index,0);
			}
			return TRUE;

		case IDC_COLORSCHEME_LIST:
			switch (HIWORD(wParam)) {
			case LBN_SELCHANGE:
				{
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int SelCount=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETSELCOUNT,0,0);
					int i;
					COLORREF SelColor=CLR_INVALID,Color;

					if (SelCount==0) {
						pThis->m_ColorPalette.SetSel(-1);
						break;
					}
					if (SelCount==1) {
						for (i=0;i<CColorScheme::NUM_COLORS;i++) {
							if ((LONG)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETSEL,i,0)>0) {
								SelColor=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETITEMDATA,i,0);
								break;
							}
						}
					} else {
						for (i=0;i<CColorScheme::NUM_COLORS;i++) {
							if ((LONG)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETSEL,i,0)>0) {
								Color=SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETITEMDATA,i,0);
								if (SelColor==CLR_INVALID)
									SelColor=Color;
								else if (Color!=SelColor)
									break;
							}
						}
						if (i<CColorScheme::NUM_COLORS) {
							pThis->m_ColorPalette.SetSel(-1);
							break;
						}
					}
					if (SelColor!=CLR_INVALID)
						pThis->m_ColorPalette.SetSel(
									pThis->m_ColorPalette.FindColor(SelColor));
				}
				break;
			}
			return TRUE;

		case IDC_COLORSCHEME_PALETTE:
			switch (HIWORD(wParam)) {
			case CColorPalette::NOTIFY_SELCHANGE:
				{
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int Sel=pThis->m_ColorPalette.GetSel();
					COLORREF Color=pThis->m_ColorPalette.GetColor(Sel);
					int i;

					for (i=0;i<CColorScheme::NUM_COLORS;i++) {
						if ((LONG)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETSEL,i,0)>0)
							SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_SETITEMDATA,i,Color);
					}
					InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);
				}
				break;
			case CColorPalette::NOTIFY_DOUBLECLICK:
				{
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int Sel=pThis->m_ColorPalette.GetSel();
					COLORREF Color=pThis->m_ColorPalette.GetColor(Sel);

					if (ChooseColorDialog(hDlg,&Color)) {
						pThis->m_ColorPalette.SetColor(Sel,Color);
						int i;

						for (i=0;i<CColorScheme::NUM_COLORS;i++) {
							if ((LONG)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETSEL,i,0)>0)
								SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_SETITEMDATA,i,Color);
						}
						InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);
					}
				}
				break;
			}
			return TRUE;

		case IDC_COLORSCHEME_PREVIEW:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);
				CColorScheme ColorScheme;

				GetListColor(hDlg,&ColorScheme);
				if (pThis->Apply(&ColorScheme))
					pThis->m_fPreview=true;
			}
			return TRUE;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case PSN_APPLY:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				GetListColor(hDlg,pThis->m_pColorScheme);
				pThis->Apply(pThis->m_pColorScheme);
			}
			break;

		case PSN_RESET:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				if (pThis->m_fPreview)
					pThis->Apply(pThis->m_pColorScheme);
			}
			break;
		}
		break;

	case WM_DESTROY:
		{
			CColorSchemeOptions *pThis=GetThis(hDlg);

			pThis->m_PresetList.Clear();
			RemoveProp(hDlg,TEXT("This"));
		}
		break;
	}
	return FALSE;
}
