#include "stdafx.h"
#include <streams.h>
#include <d3d9.h>
#include <vmr9.h>
#include "VideoRenderer.h"
#include "DirectShowUtil.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif




CVideoRenderer::CVideoRenderer()
{
	m_pRenderer=NULL;
	m_pFilterGraph=NULL;
	m_hwndRender=NULL;
}


CVideoRenderer::~CVideoRenderer()
{
	TRACE(TEXT("CVideoRenderer::~CVideoRenderer()\n"));
}


bool CVideoRenderer::ShowProperty(HWND hwndOwner)
{
	if (m_pRenderer)
		return CDirectShowUtil::ShowPropertyPage(m_pRenderer,hwndOwner);
	return false;
}




class CVideoRenderer_Default : public CVideoRenderer {
	IVideoWindow *m_pVideoWindow;
	IBasicVideo2 *m_pBasicVideo;
public:
	CVideoRenderer_Default();
	bool Initialize(IGraphBuilder *pFilterGraph,IPin *pInputPin,HWND hwndRender,HWND hwndMessageDrain);
	bool Finalize();
	bool SetVideoPosition(int SourceWidth,int SourceHeight,const RECT *pSourceRect,
								const RECT *pDestRect,const RECT *pWindowRect);
	bool GetDestPosition(RECT *pRect);
	bool GetCurrentImage(void **ppBuffer);
	bool ShowCursor(bool fShow);
};


CVideoRenderer_Default::CVideoRenderer_Default()
{
	m_pVideoWindow=NULL;
	m_pBasicVideo=NULL;
}


bool CVideoRenderer_Default::Initialize(IGraphBuilder *pFilterGraph,IPin *pInputPin,HWND hwndRender,HWND hwndMessageDrain)
{
	HRESULT hr;
	RECT rc;

	hr=pFilterGraph->Render(pInputPin);
	if (FAILED(hr)) {
		return false;
	}
	hr=pFilterGraph->QueryInterface(IID_IVideoWindow,reinterpret_cast<LPVOID *>(&m_pVideoWindow));
	if (FAILED(hr)) {
		return false;
	}
	hr=pFilterGraph->QueryInterface(IID_IBasicVideo2,reinterpret_cast<LPVOID *>(&m_pBasicVideo));
	if (FAILED(hr)) {
		SAFE_RELEASE(m_pVideoWindow);
		return false;
	}
	m_pVideoWindow->put_Owner((OAHWND)hwndRender);
	m_pVideoWindow->put_MessageDrain((OAHWND)hwndMessageDrain);
	m_pVideoWindow->put_WindowStyle(WS_CHILD | WS_CLIPSIBLINGS);
	m_pVideoWindow->put_BackgroundPalette(OATRUE);
	m_pVideoWindow->put_BorderColor(RGB(0,0,0));
	m_pVideoWindow->put_Caption(L"");
	::GetClientRect(hwndRender,&rc);
	m_pVideoWindow->SetWindowPosition(0,0,rc.right,rc.bottom);
	m_pVideoWindow->SetWindowForeground(OATRUE);
	m_pVideoWindow->put_Visible(OATRUE);
	m_pFilterGraph=pFilterGraph;
	return true;
}


bool CVideoRenderer_Default::Finalize()
{
	SAFE_RELEASE(m_pBasicVideo);
	if (m_pVideoWindow) {
		m_pVideoWindow->put_Visible(OAFALSE);
		m_pVideoWindow->put_Owner(NULL);
		SAFE_RELEASE(m_pVideoWindow);
	}
	return true;
}


bool CVideoRenderer_Default::SetVideoPosition(int SourceWidth,int SourceHeight,const RECT *pSourceRect,
								const RECT *pDestRect,const RECT *pWindowRect)
{
	HRESULT hr;

	if (m_pVideoWindow==NULL || m_pBasicVideo==NULL)
		return false;
	m_pBasicVideo->SetSourcePosition(pSourceRect->left,pSourceRect->top,
		pSourceRect->right-pSourceRect->left,pSourceRect->bottom-pSourceRect->top);
	m_pBasicVideo->SetDestinationPosition(pDestRect->left,pDestRect->top,
		pDestRect->right-pDestRect->left,pDestRect->bottom-pDestRect->top);
	m_pVideoWindow->SetWindowPosition(pWindowRect->left,pWindowRect->top,
		pWindowRect->right-pWindowRect->left,pWindowRect->bottom-pWindowRect->top);
	return true;
}


bool CVideoRenderer_Default::GetDestPosition(RECT *pRect)
{
	if (m_pBasicVideo==NULL
			|| FAILED(m_pBasicVideo->GetDestinationPosition(
					&pRect->left,&pRect->top,&pRect->right,&pRect->bottom)))
		return false;
	pRect->right+=pRect->left;
	pRect->bottom+=pRect->top;
	return true;
}


bool CVideoRenderer_Default::GetCurrentImage(void **ppBuffer)
{
	bool fOK=false;

	if (m_pBasicVideo!=NULL) {
		long Size;

		if (SUCCEEDED(m_pBasicVideo->GetCurrentImage(&Size,NULL)) && Size>0) {
			long *pDib;

			pDib=static_cast<long*>(::CoTaskMemAlloc(Size));
			if (pDib!=NULL) {
				if (SUCCEEDED(m_pBasicVideo->GetCurrentImage(&Size,pDib))) {
					*ppBuffer=pDib;
					fOK=true;
				} else {
					::CoTaskMemFree(pDib);
				}
			}
		}
	}
	return fOK;
}


bool CVideoRenderer_Default::ShowCursor(bool fShow)
{
	if (m_pVideoWindow) {
		if (SUCCEEDED(m_pVideoWindow->HideCursor(fShow?OAFALSE:OATRUE)))
			return true;
	}
	return false;
}




class CVideoRenderer_VMR7 : public CVideoRenderer {
public:
	CVideoRenderer_VMR7();
	bool Initialize(IGraphBuilder *pFilterGraph,IPin *pInputPin,HWND hwndRender,HWND hwndMessageDrain);
	bool Finalize();
	bool SetVideoPosition(int SourceWidth,int SourceHeight,const RECT *pSourceRect,
								const RECT *pDestRect,const RECT *pWindowRect);
	bool GetDestPosition(RECT *pRect);
	bool GetCurrentImage(void **ppBuffer);
	bool RepaintVideo(HWND hwnd,HDC hdc);
	bool DisplayModeChanged();
};


CVideoRenderer_VMR7::CVideoRenderer_VMR7()
{
}


bool CVideoRenderer_VMR7::Initialize(IGraphBuilder *pFilterGraph,IPin *pInputPin,HWND hwndRender,HWND hwndMessageDrain)
{
	HRESULT hr;

	hr=::CoCreateInstance(CLSID_VideoMixingRenderer,NULL,CLSCTX_INPROC_SERVER,
						IID_IBaseFilter,reinterpret_cast<LPVOID*>(&m_pRenderer));
	if (FAILED(hr))
		return false;
	hr=pFilterGraph->AddFilter(m_pRenderer,L"VMR7");
	if (FAILED(hr)) {
		SAFE_RELEASE(m_pRenderer);
		return false;
	}

	IVMRFilterConfig *pFilterConfig;
	IVMRWindowlessControl *pWindowlessControl;
	RECT rc;

	m_pRenderer->QueryInterface(IID_IVMRFilterConfig,
									reinterpret_cast<LPVOID*>(&pFilterConfig));
	pFilterConfig->SetRenderingMode(VMRMode_Windowless);
	pFilterConfig->Release();
	m_pRenderer->QueryInterface(IID_IVMRWindowlessControl,
							reinterpret_cast<LPVOID*>(&pWindowlessControl));
	pWindowlessControl->SetVideoClippingWindow(hwndRender);
	pWindowlessControl->SetBorderColor(RGB(0,0,0));
	pWindowlessControl->SetAspectRatioMode(VMR9ARMode_None);
	::GetClientRect(hwndRender,&rc);
	pWindowlessControl->SetVideoPosition(NULL,&rc);
	pWindowlessControl->Release();

	IFilterGraph2 *pFilterGraph2;

	hr=pFilterGraph->QueryInterface(IID_IFilterGraph2,
									reinterpret_cast<LPVOID*>(&pFilterGraph2));
	if (FAILED(hr)) {
		return false;
	}
	hr=pFilterGraph2->RenderEx(pInputPin,
								AM_RENDEREX_RENDERTOEXISTINGRENDERERS,NULL);
	pFilterGraph2->Release();
	if (FAILED(hr)) {
		return false;
	}
	m_pFilterGraph=pFilterGraph;
	m_hwndRender=hwndRender;
	return true;
}


bool CVideoRenderer_VMR7::Finalize()
{
	SAFE_RELEASE(m_pRenderer);
	return true;
}


bool CVideoRenderer_VMR7::SetVideoPosition(int SourceWidth,int SourceHeight,const RECT *pSourceRect,
								const RECT *pDestRect,const RECT *pWindowRect)
{
	IVMRWindowlessControl *pWindowlessControl;
	HRESULT hr;
	RECT rcSrc;

	if (m_pRenderer==NULL)
		return false;
	hr=m_pRenderer->QueryInterface(IID_IVMRWindowlessControl,
							reinterpret_cast<LPVOID*>(&pWindowlessControl));
	if (FAILED(hr))
		return false;

	LONG Width,Height;

	if (SUCCEEDED(pWindowlessControl->GetNativeVideoSize(&Width,&Height,NULL,NULL))
			&& SourceWidth>0 && SourceHeight>0) {
		rcSrc.left=pSourceRect->left*Width/SourceWidth;
		rcSrc.top=pSourceRect->top*Height/SourceHeight;
		rcSrc.right=pSourceRect->right*Width/SourceWidth;
		rcSrc.bottom=pSourceRect->bottom*Height/SourceHeight;
	} else {
		rcSrc=*pSourceRect;
	}
	// Ȃ const RECT* ł͂Ȃ const LPRECT ɂȂĂ...
	pWindowlessControl->SetVideoPosition(&rcSrc,const_cast<LPRECT>(pDestRect));
	pWindowlessControl->Release();
	InvalidateRect(m_hwndRender,NULL,TRUE);
	return true;
}


bool CVideoRenderer_VMR7::GetDestPosition(RECT *pRect)
{
	bool fOK=false;

	if (m_pRenderer) {
		IVMRWindowlessControl *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			fOK=SUCCEEDED(pWindowlessControl->GetVideoPosition(NULL,pRect));
			pWindowlessControl->Release();
		}
	}
	return fOK;
}


bool CVideoRenderer_VMR7::GetCurrentImage(void **ppBuffer)
{
	bool fOK=false;

	if (m_pRenderer) {
		IVMRWindowlessControl *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			BYTE *pDib;

			if (SUCCEEDED(pWindowlessControl->GetCurrentImage(&pDib))) {
				*ppBuffer=pDib;
				fOK=true;
			}
			pWindowlessControl->Release();
		}
	}
	return fOK;
}


bool CVideoRenderer_VMR7::RepaintVideo(HWND hwnd,HDC hdc)
{
	bool fOK=false;

	if (m_pRenderer) {
		IVMRWindowlessControl *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			if (SUCCEEDED(pWindowlessControl->RepaintVideo(hwnd,hdc)))
				fOK=true;
			pWindowlessControl->Release();
		}
	}
	return fOK;
}


bool CVideoRenderer_VMR7::DisplayModeChanged()
{
	bool fOK=false;

	if (m_pRenderer) {
		IVMRWindowlessControl *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			if (SUCCEEDED(pWindowlessControl->DisplayModeChanged()))
				fOK=true;
			pWindowlessControl->Release();
		}
	}
	return fOK;
}




class CVideoRenderer_VMR9 : public CVideoRenderer {
public:
	CVideoRenderer_VMR9();
	bool Initialize(IGraphBuilder *pFilterGraph,IPin *pInputPin,HWND hwndRender,HWND hwndMessageDrain);
	bool Finalize();
	virtual bool SetVideoPosition(int SourceWidth,int SourceHeight,const RECT *pSourceRect,
								const RECT *pDestRect,const RECT *pWindowRect);
	bool GetDestPosition(RECT *pRect);
	bool GetCurrentImage(void **ppBuffer);
	bool RepaintVideo(HWND hwnd,HDC hdc);
	bool DisplayModeChanged();
};


CVideoRenderer_VMR9::CVideoRenderer_VMR9()
{
}


bool CVideoRenderer_VMR9::Initialize(IGraphBuilder *pFilterGraph,IPin *pInputPin,HWND hwndRender,HWND hwndMessageDrain)
{
	HRESULT hr;

	hr=::CoCreateInstance(CLSID_VideoMixingRenderer9,NULL,CLSCTX_INPROC_SERVER,
						IID_IBaseFilter,reinterpret_cast<LPVOID*>(&m_pRenderer));
	if (FAILED(hr))
		return false;
	hr=pFilterGraph->AddFilter(m_pRenderer,L"VMR9");
	if (FAILED(hr)) {
		SAFE_RELEASE(m_pRenderer);
		return false;
	}

	IVMRFilterConfig *pFilterConfig;
	IVMRWindowlessControl9 *pWindowlessControl;
	IVMRMixerControl9 *pMixerControl;
	RECT rc;

	m_pRenderer->QueryInterface(IID_IVMRFilterConfig9,
									reinterpret_cast<LPVOID*>(&pFilterConfig));
	pFilterConfig->SetRenderingMode(VMRMode_Windowless);
	pFilterConfig->Release();
	m_pRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl));
	pWindowlessControl->SetVideoClippingWindow(hwndRender);
	pWindowlessControl->SetBorderColor(RGB(0,0,0));
	pWindowlessControl->SetAspectRatioMode(VMR9ARMode_None);
	::GetClientRect(hwndRender,&rc);
	pWindowlessControl->SetVideoPosition(NULL,&rc);
	pWindowlessControl->Release();
	if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRMixerControl9,
								reinterpret_cast<LPVOID*>(&pMixerControl)))) {
		DWORD MixingPref;

		pMixerControl->GetMixingPrefs(&MixingPref);
		MixingPref=(MixingPref&~MixerPref9_DecimateMask)|MixerPref9_NonSquareMixing;
		pMixerControl->SetMixingPrefs(MixingPref);
		pMixerControl->Release();
	}

	IFilterGraph2 *pFilterGraph2;

	hr=pFilterGraph->QueryInterface(IID_IFilterGraph2,
									reinterpret_cast<LPVOID*>(&pFilterGraph2));
	if (FAILED(hr)) {
		return false;
	}
	hr=pFilterGraph2->RenderEx(pInputPin,
								AM_RENDEREX_RENDERTOEXISTINGRENDERERS,NULL);
	pFilterGraph2->Release();
	if (FAILED(hr)) {
		return false;
	}
	m_pFilterGraph=pFilterGraph;
	m_hwndRender=hwndRender;
	return true;
}


bool CVideoRenderer_VMR9::Finalize()
{
	SAFE_RELEASE(m_pRenderer);
	return true;
}


bool CVideoRenderer_VMR9::SetVideoPosition(int SourceWidth,int SourceHeight,const RECT *pSourceRect,
								const RECT *pDestRect,const RECT *pWindowRect)
{
	IVMRWindowlessControl9 *pWindowlessControl;
	HRESULT hr;
	RECT rcSrc;

	if (m_pRenderer==NULL)
		return false;
	hr=m_pRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl));
	if (FAILED(hr))
		return false;

	LONG Width,Height;

	if (SUCCEEDED(pWindowlessControl->GetNativeVideoSize(&Width,&Height,NULL,NULL))
			&& SourceWidth>0 && SourceHeight>0) {
		rcSrc.left=pSourceRect->left*Width/SourceWidth;
		rcSrc.top=pSourceRect->top*Height/SourceHeight;
		rcSrc.right=pSourceRect->right*Width/SourceWidth;
		rcSrc.bottom=pSourceRect->bottom*Height/SourceHeight;
	} else {
		rcSrc=*pSourceRect;
	}
	// Ȃ const RECT* ł͂Ȃ const LPRECT ɂȂĂ...
	pWindowlessControl->SetVideoPosition(&rcSrc,const_cast<LPRECT>(pDestRect));
	pWindowlessControl->Release();
	InvalidateRect(m_hwndRender,NULL,TRUE);
	return true;
}


bool CVideoRenderer_VMR9::GetDestPosition(RECT *pRect)
{
	bool fOK=false;

	if (m_pRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			fOK=SUCCEEDED(pWindowlessControl->GetVideoPosition(NULL,pRect));
			pWindowlessControl->Release();
		}
	}
	return fOK;
}


bool CVideoRenderer_VMR9::GetCurrentImage(void **ppBuffer)
{
	bool fOK=false;

	if (m_pRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			BYTE *pDib;

			if (SUCCEEDED(pWindowlessControl->GetCurrentImage(&pDib))) {
				*ppBuffer=pDib;
				fOK=true;
			}
			pWindowlessControl->Release();
		}
	}
	return fOK;
}


bool CVideoRenderer_VMR9::RepaintVideo(HWND hwnd,HDC hdc)
{
	bool fOK=false;

	if (m_pRenderer!=NULL) {
		IVMRWindowlessControl9 *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			if (SUCCEEDED(pWindowlessControl->RepaintVideo(hwnd,hdc)))
				fOK=true;
			pWindowlessControl->Release();
		}
	}
	return fOK;
}


bool CVideoRenderer_VMR9::DisplayModeChanged()
{
	bool fOK=false;

	if (m_pRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;

		if (SUCCEEDED(m_pRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			if (SUCCEEDED(pWindowlessControl->DisplayModeChanged()))
				fOK=true;
			pWindowlessControl->Release();
		}
	}
	return fOK;
}




bool CVideoRenderer::CreateRenderer(RendererType Type,CVideoRenderer **ppRenderer)
{
	switch (Type) {
	case RENDERER_DEFAULT:
		*ppRenderer=new CVideoRenderer_Default;
		break;
	case RENDERER_VMR7:
		*ppRenderer=new CVideoRenderer_VMR7;
		break;
	case RENDERER_VMR9:
		*ppRenderer=new CVideoRenderer_VMR9;
		break;
	default:
		return false;
	}
	return true;
}


LPCTSTR CVideoRenderer::EnumRendererName(int Index)
{
	static const LPCTSTR pszRendererName[] = {
		TEXT("Default"),
		TEXT("VMR7"),
		TEXT("VMR9"),
	};

	if (Index<0 || Index>=3)
		return NULL;
	return pszRendererName[Index];
}


CVideoRenderer::RendererType CVideoRenderer::ParseName(LPCTSTR pszName)
{
	LPCTSTR pszRenderer;
	int i;

	for (i=0;(pszRenderer=EnumRendererName(i))!=NULL;i++) {
		if (lstrcmpi(pszName,pszRenderer)==0)
			return (RendererType)i;
	}
	return RENDERER_UNDEFINED;
}
