#ifndef EPG_PROGRAM_LIST
#define EPG_PROGRAM_LIST


#include <vector>
#include <map>
#include "Epg.h"


class CServiceInfoData {
	LPWSTR m_pszServiceName;
public:
	WORD m_OriginalNID;
	WORD m_TSID;
	WORD m_ServiceID;
	WORD m_ServiceType;
	CServiceInfoData();
	CServiceInfoData(const CServiceInfoData &Info);
	CServiceInfoData(WORD OriginalNID,WORD TSID,WORD ServiceID,WORD ServiceType,LPCWSTR pszServiceName);
	~CServiceInfoData();
	CServiceInfoData &operator=(const CServiceInfoData &Info);
	LPCWSTR GetServiceName() const { return m_pszServiceName; }
	bool SetServiceName(LPCWSTR pszName);
};

struct NibbleData {
	BYTE m_ContentNibbleLv1;	//content_nibble_level_1
	BYTE m_ContentNibbleLv2;	//content_nibble_level_2
	BYTE m_UserNibbleLv1;		//user_nibble
	BYTE m_UserNibbleLv2;		//user_nibble
};

class CEventInfoData {
	LPWSTR m_pszEventName;
	LPWSTR m_pszEventText;
	LPWSTR m_pszEventExtText;
	LPWSTR m_pszComponentTypeText;
	LPWSTR m_pszAudioComponentTypeText;
public:
	enum {
		AUDIOCOMPONENT_MONO=1,
		AUDIOCOMPONENT_DUALMONO,
		AUDIOCOMPONENT_STEREO,
		AUDIOCOMPONENT_3_1,
		AUDIOCOMPONENT_3_2,
		AUDIOCOMPONENT_3_2_1
	};
	WORD m_OriginalNID;
	WORD m_TSID;
	WORD m_ServiceID;
	WORD m_EventID;
	SYSTEMTIME m_stStartTime;
	DWORD m_DurationSec;
	BYTE m_ComponentType;
	BYTE m_AudioComponentType;
	BYTE m_ESMultiLangFlag;
	BYTE m_MainComponentFlag;
	BYTE m_SamplingRate;
	std::vector<NibbleData> m_NibbleList;
	CEventInfoData();
	CEventInfoData(const CEventInfoData &Info);
	~CEventInfoData();
	CEventInfoData &operator=(const CEventInfoData &Info);
	LPCWSTR GetEventName() const { return m_pszEventName; }
	bool SetEventName(LPCWSTR pszEventName);
	LPCWSTR GetEventText() const { return m_pszEventText; }
	bool SetEventText(LPCWSTR pszEventText);
	LPCWSTR GetEventExtText() const { return m_pszEventExtText; }
	bool SetEventExtText(LPCWSTR pszEventExtText);
	LPCWSTR GetComponentTypeText() const { return m_pszComponentTypeText; }
	bool SetComponentTypeText(LPCWSTR pszText);
	LPCWSTR GetAudioComponentTypeText() const { return m_pszAudioComponentTypeText; }
	bool SetAudioComponentTypeText(LPCWSTR pszText);
};

class CEventInfoList {
public:
	std::map<WORD,CEventInfoData> EventDataMap; //L[ EventID
	CEventInfoList & operator= (const CEventInfoList & o) {
		EventDataMap = o.EventDataMap;
		return *this;
	};
};

class CEpgProgramList {
	CEpgDataCapDllUtil *m_pEpgDll;
public:
	typedef std::map<ULONGLONG,CServiceInfoData>::iterator ServiceIterator;
	typedef std::map<ULONGLONG,CEventInfoList>::iterator EventListIterator;
	typedef std::map<WORD,CEventInfoData>::iterator EventIterator;
	std::map<ULONGLONG,CServiceInfoData> ServiceMap;
	std::map<ULONGLONG,CEventInfoList> EventMap;
	CEpgProgramList();
	~CEpgProgramList();
	void SetEpgDataCapDllUtil(CEpgDataCapDllUtil *pEpgDll) { m_pEpgDll=pEpgDll; }
	bool UpdateProgramList();
	void Clear();
	bool GetEventInfo(WORD ServiceID,WORD EventID,CEventInfoData *pInfo);
};


#endif
