#include "stdafx.h"
#include <commctrl.h>
#include "HDUSTest.h"
#include "Logger.h"
#include "resource.h"




CLogItem::CLogItem(LPCTSTR pszText)
{
	::GetSystemTimeAsFileTime(&m_Time);
	m_pszText=DuplicateString(pszText);
}


CLogItem::~CLogItem()
{
	delete [] m_pszText;
}


void CLogItem::GetTime(SYSTEMTIME *pTime) const
{
	SYSTEMTIME stUTC;

	::FileTimeToSystemTime(&m_Time,&stUTC);
	::SystemTimeToTzSpecificLocalTime(NULL,&stUTC,pTime);
}




CLogger::CLogger()
{
	m_NumLogItems=0;
	m_ppList=NULL;
	m_ListLength=0;
}


CLogger::~CLogger()
{
	Clear();
}


bool CLogger::AddLog(LPCTSTR pszText, ...)
{
	if (m_NumLogItems==m_ListLength) {
		if (m_ListLength==0)
			m_ListLength=64;
		else
			m_ListLength*=2;
		m_ppList=static_cast<CLogItem**>(realloc(m_ppList,m_ListLength*sizeof(CLogItem*)));
	}

	va_list Args;
	TCHAR szText[1024];

	va_start(Args,pszText);
	_vsntprintf(szText,lengthof(szText),pszText,Args);
	m_ppList[m_NumLogItems++]=new CLogItem(szText);
	va_end(Args);
	return true;
}


void CLogger::Clear()
{
	if (m_ppList!=NULL) {
		int i;

		for (i=m_NumLogItems-1;i>=0;i--)
			delete m_ppList[i];
		free(m_ppList);
		m_ppList=NULL;
		m_NumLogItems=0;
		m_ListLength=0;
	}
}


CLogger *CLogger::GetThis(HWND hDlg)
{
	return static_cast<CLogger*>(::GetProp(hDlg,TEXT("This")));
}


BOOL CALLBACK CLogger::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			CLogger *pThis=reinterpret_cast<CLogger*>(lParam);
			HWND hwndList=GetDlgItem(hDlg,IDC_LOG_LIST);
			LV_COLUMN lvc;
			LV_ITEM lvi;
			int i;

			::SetProp(hDlg,TEXT("This"),pThis);
			lvc.mask=LVCF_FMT | LVCF_WIDTH | LVCF_TEXT;
			lvc.fmt=LVCFMT_LEFT;
			lvc.cx=80;
			lvc.pszText=TEXT("");
			ListView_InsertColumn(hwndList,0,&lvc);
			lvc.pszText=TEXT("e");
			ListView_InsertColumn(hwndList,1,&lvc);
			lvi.mask=LVIF_TEXT;
			for (i=0;i<pThis->m_NumLogItems;i++) {
				SYSTEMTIME st;
				int Length;
				TCHAR szTime[64];

				lvi.iItem=i;
				lvi.iSubItem=0;
				pThis->m_ppList[i]->GetTime(&st);
				Length=::GetDateFormat(LOCALE_USER_DEFAULT,DATE_SHORTDATE,&st,
											NULL,szTime,lengthof(szTime)-1);
				szTime[Length-1]=' ';
				::GetTimeFormat(LOCALE_USER_DEFAULT,TIME_FORCE24HOURFORMAT,&st,
								NULL,szTime+Length,lengthof(szTime)-Length);
				lvi.pszText=szTime;
				ListView_InsertItem(hwndList,&lvi);
				lvi.iSubItem=1;
				lvi.pszText=(LPTSTR)pThis->m_ppList[i]->GetText();
				ListView_SetItem(hwndList,&lvi);
			}
			for (i=0;i<2;i++)
				ListView_SetColumnWidth(hwndList,i,LVSCW_AUTOSIZE_USEHEADER);
			if (pThis->m_NumLogItems>0)
				ListView_EnsureVisible(hwndList,pThis->m_NumLogItems-1,FALSE);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_LOG_CLEAR:
			{
				CLogger *pThis=GetThis(hDlg);

				ListView_DeleteAllItems(GetDlgItem(hDlg,IDC_LOG_LIST));
				pThis->Clear();
			}
			return TRUE;
		}
		return TRUE;

	case WM_DESTROY:
		::RemoveProp(hDlg,TEXT("This"));
		return TRUE;
	}
	return FALSE;
}
